/*
    Copyright 2008-2009 Web-Mining.fr
    Authors : Mathieu Bastian, Damien Bastian
    Website : http://timmyminer.mozdev.org

    This file is part of Timmy Miner.
	
    Timmy Miner is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

   Timmy Miner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Timmy Miner.  If not, see <http://www.gnu.org/licenses/>.
*/


/*+--------------------------------------------------------------------+
    |				    INIT		 		|
    +------------------------------------------------------------- ------+*/

const STATE_GREEN = 0;
const STATE_RED = 1;
	
//Instantiate objects
var tymining_RunController				= new tymining_RunController();
var tymining_PreferencesController 		= new tymining_PreferencesController();
var tymining_ErrorController			= new tymining_ErrorController();
var tymining_DataSourceController		= new tymining_DataSourceController();
var tymining_XPCOMController			= new tymining_XPCOMController();
var tymining_UIController				= new tymining_UIController();
var tymining_DataController				= new tymining_DataController();
var tymining_ExportController			= new tymining_ExportController();

//Locale
var tymining_StringBundle = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
var tymining_Locale	= tymining_StringBundle.createBundle("chrome://tymining/locale/tymining.properties");

//Init listener
function tymining_listeners() 
{
	if(document.getElementById("navigator-toolbox") == null)
		return;
	

	//Init
	tymining_XPCOMController.initObservers();
	LOG("Timmy Miner loaded");
	
	//Be sure the listener is added only once
	window.removeEventListener("load", tymining_listeners, true);
}

window.addEventListener("load", tymining_listeners, true);


/*+--------------------------------------------------------------------+
    |		             PAGE LOADING		 		|
    +------------------------------------------------------------- ------+*/

// Content loading listener
function tymining_onContentLoaded(aEvent)
{
	if(tymining_RunController.IS_RUNNING())
	{
	
		//Only HTML
		if (aEvent.originalTarget instanceof HTMLDocument && (aEvent.originalTarget.contentType == "text/html" || aEvent.originalTarget.contentType == "application/xhtml+xml"))
		{
			var doc = aEvent.originalTarget;
			
			//Frame Event
			if (aEvent.originalTarget.defaultView.frameElement && tymining_PreferencesController.prefs.getBoolPref("extensions.tymining.IncludeFrameLoadingEvent")) 
			{
				//Get original document
				while (doc.defaultView.frameElement) 
				{
					doc=doc.defaultView.frameElement.ownerDocument;
				}
			}
			else //Normal origin
			{

				if(doc.location == "about:blank")
					return 0;
					
				var validURL = true;	
				var filtering = tymining_PreferencesController.prefs.getBoolPref("extensions.tymining.FilterURLs");
				if(filtering)
				{
					var theTab = tymining_getTabFromDocument(aEvent.originalTarget);
					if(theTab.currentURI == null || theTab.currentURI == undefined)
					{
						validURL=false;
					}
				}

				if(validURL)
				{
					var url = doc.location
					var pageId = tymining_DataController.addPage(url);
					tymining_XPCOMController.analysePage(pageId, doc);
				}
			}	
		}
	} 	
		
}

function tymining_getTabFromDocument(theDocument)
{
	var theTabbrowser = this.getBrowser();
	if (theTabbrowser.mTabbedMode) { 
		var browserIndex = theTabbrowser.getBrowserIndexForDocument(theDocument); 
		if (browserIndex == -1){ 
			//alert("Erreur lors de la dtermination de l'onglet"); 
			return theDocument;
		} 
		return theTabbrowser.getBrowserAtIndex(browserIndex); 
	}
	//alert("Mode sans onglet"); 
	return theTabbrowser.selectedBrowser;
}


/*+--------------------------------------------------------------------+
    |		            DATA OBJECTS		 		|
    +------------------------------------------------------------- ------+*/

function TyminingPage(_id,_url)
{
	this.id = _id;
	this.url = _url;
	this.lang = 0;
	this.state = STATE_GREEN;
}


/*+--------------------------------------------------------------------+
    |			   CONTROLLERS		 		|
    +------------------------------------------------------------- ------+*/

	
//XPCOM Controller
function tymining_XPCOMController()
{

	this.initComponent = function()
	{
		var iface = Components.interfaces.nsITymining;
		var cppClass = Components.classes["@web-mining.fr/tymining;1"];

		var cppComponent = cppClass.createInstance();
		if (cppComponent) 
		{
			cppComponent = cppComponent.QueryInterface(Components.interfaces.nsITymining);
			cppComponent = cppComponent.QueryInterface(iface);
			this.compo = cppComponent;

			cppComponent.initAnalyser();
		}
		else
		{
			tymining_ErrorController.Severe("Impossible to instanciate the XPCOM Component");
		}
	}
	
	this.getComponent = function()
	{
		return this.compo;
	}
	
	this.initObservers = function()
	{
		//Observeur
		var ObserverServ = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
		// enregistrement
		ObserverServ.addObserver (this, "tymining_inited" , false );
		ObserverServ.addObserver (this, "tymining_initlang" , false );
		ObserverServ.addObserver (this, "tymining_page_analysed" , false );
		ObserverServ.addObserver (this, "tymining_log" , false );
		ObserverServ.addObserver (this, "tymining_progress" , false );
		ObserverServ.addObserver (this, "tymining_refreshwordgraph" , false );
	}
	
	this.analysisCount=0;
	this.analysePage = function(pageId, doc)
	{
		if(this.analysisCount==0)
		{
			//First analyse
			tymining_UIController.enableAnalysis();
		}
		this.analysisCount++;
		this.compo.setPageDocument(pageId, doc);
	}
	
	this.initCount=0;
	this.observe = function (subject, topic, data)
	{
		switch (topic) {
			case 'tymining_log':
				tymining_UIController.appendLog(""+data);
				break;
			case 'tymining_progress':
				var progress = 1*data;
				tymining_UIController.setProgress(progress);
				break;
			case 'tymining_inited':
				this.initCount++;
				if(this.initCount==2)
				{
					this.initCount=0;
					
					//Create corpus
					var minOcc = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.CorpusMinWords");
					this.compo.createCorpus(minOcc);
					
					tymining_RunController.INITED();
					tymining_UIController.startBtn();
					tymining_UIController.initLangOptions();
				}
				break;
			case 'tymining_initlang':
				tymining_DataController.addLang(""+data);
				tymining_UIController.appendLog("--stoplist: "+data);
				break;
			case 'tymining_page_analysed':
				var pageId = data*1;
				var pageObject = tymining_DataController.getPage(pageId);
				
				//PAGE
				tymining_XPCOMController.pageTreatment(pageObject);
				
				//CORPUS
				tymining_XPCOMController.corpusTreatment();
				
				//AnalyseCount
				this.analysisCount--;
				if(this.analysisCount==0)
				{
					//No more analyse running
					tymining_UIController.disableAnalysis();
				}
				break;
			case 'tymining_refreshwordgraph':
				tymining_ExportController.exportWordGraphStep2();
				break;
		}
	}
	
	this.pageTreatment = function(pageObject)
	{
		if(tymining_UIController.ui)
		{
			var result = {value: null};
			this.compo.getPageCorpus(pageObject.id,-1, result);
			var xmlResult = result.value;
			tymining_DataSourceController.transformPageXML(xmlResult);
			//LOG(""+xmlResult);
					
			//Get lang and state
			result = {value: null};
			this.compo.getPageLang(pageObject.id, result)
			var lang = result.value;
			result = {value: null};
			this.compo.getPageState(pageObject.id,result)
			var state = result.value;
			pageObject.lang = lang;
			pageObject.state = state;
					
			//Refresh page info
			tymining_UIController.refreshPage(pageObject);
		}
		else
		{
			//Current page
			this.currentPage=pageObject;
		}	
	}
	
	this.corpusTreatment = function()
	{
		if(tymining_UIController.ui)
		{
			result = {value: null};
			this.compo.getAllCorpus(-1, result);
			xmlResult = result.value;
			tymining_DataSourceController.transformCorpusXML(xmlResult);
			//LOG(""+xmlResult);
						
			//Get corpus info in xml
						
						
			//Refresh corpus info
			tymining_UIController.refreshCorpus();
		}
	}
	
	this.isAnalysing = function()
	{
		return this.analysisCount > 0;
	}
	
	this.addPage = function(pageObject)
	{
		var pageId = pageObject.id;
		var pageUrl = pageObject.url;
		var state = pageObject.state;
		this.compo.addPage(pageId, pageUrl, state);
		
		if(!tymining_UIController.langAuto)
		{
			LOG("setpage "+tymining_UIController.defaultLang);
			this.compo.setPageLang(pageId, tymining_UIController.defaultLang);
		}
	}
	
	
	this.setPageState = function(pageObject)
	{
		this.compo.setPageState(pageObject.id, pageObject.state);
	}
	
	this.reset = function()
	{
		//Create corpus
		var minOcc = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.CorpusMinWords");
		this.compo.createCorpus(minOcc);
	}

}



//DataSourceController
function tymining_DataSourceController()
{
	this.createDataSource = function(str)
	{
		var ds = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"].createInstance(Components.interfaces.nsIRDFDataSource);
		
		var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var baseUri = ios.newURI("http://www.web-mining.fr", null, null);
		
		//var str = "<?xml version=\"1.0\"?><RDF:RDF xmlns:RDF=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:row=\"http://www.web-mining.fr/rdf#\"><RDF:Seq RDF:about=\"http://www.web-mining.fr/root\"><RDF:li><RDF:Description><row:word>Lion</row:word><row:occ>Panthera leo</row:occ></RDF:Description></RDF:li><RDF:li><RDF:Description><row:word>Poney</row:word><row:occ>Caca</row:occ></RDF:Description></RDF:li></RDF:Seq></RDF:RDF>";

		var parser = Components.classes["@mozilla.org/rdf/xml-parser;1"].createInstance(Components.interfaces.nsIRDFXMLParser);
		parser.parseString(ds, baseUri, str);
		
		return ds;
	}
	
	this.assignedDatasourcePage=0;
	this.assignedDatasourceCorpus=0;
	this.assignDataSource = function(domId, datasource, oldDatasource)
	{
		var tree = tymining_UIController.getSidebar().document.getElementById(domId);
		if(oldDatasource!=0)
			tree.database.RemoveDataSource(oldDatasource);
		tree.database.AddDataSource(datasource);
		tree.setAttribute("ref", "http://www.web-mining.fr/root");
		tree.builder.rebuild();
	}
	
	this.removeDataSource = function(domId, datasource)
	{
		var tree = tymining_UIController.getSidebar().document.getElementById(domId);
		if(datasource!=0)
			tree.database.RemoveDataSource(datasource);
		tree.builder.rebuild();
	}
	
	this.pageXSL = 0;
	this.corpusXSL = 0;
	this.initXSLT = function()
	{
		//XSLT doc page
		var xsltDoc = document.implementation.createDocument("", "test", null);
		this.pageXSL = xsltDoc;
		xsltDoc.addEventListener("load", tymining_DataSourceController.initXSLTFinished, false);
		xsltDoc.load("chrome://tymining/content/page.xsl");
		
		//XSLT doc corpus
		var xsltDoc2 = document.implementation.createDocument("", "test", null);
		this.corpusXSL = xsltDoc2;
		xsltDoc2.addEventListener("load", tymining_DataSourceController.initXSLTFinished, false);
		xsltDoc2.load("chrome://tymining/content/corpus.xsl");
	}
	
	this.initXSLTFinished = function()
	{
		
	}
	
	
	this.transformPageXML = function(xmltxt)
	{	
			var iosvc = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
			var aboutBlankURI = iosvc.newURI("about:blank", null, null); 
			var parser = new DOMParser(null,aboutBlankURI,null);
            var xmlDocument = parser.parseFromString(xmltxt, "text/xml");
			var roottag = xmlDocument.documentElement;
		
			//Detection erreur du Parser
			if ((roottag.tagName == "parserError") || (roottag.namespaceURI == "http://www.mozilla.org/newlayout/xml/parsererror.xml"))
			{
				alert("Erreur XML");
			}
			
            var xsltProcessor = new XSLTProcessor();
            xsltProcessor.importStylesheet(this.pageXSL);
			var resultDocument = xsltProcessor.transformToDocument(xmlDocument);
			

			/* on transforme le doc XML en une string */
			var str = new XMLSerializer().serializeToString(resultDocument);
			var ds = tymining_DataSourceController.createDataSource(str);
			tymining_DataSourceController.assignDataSource("tymining_pagetree",ds, this.assignedDatasourcePage);
			this.assignedDatasourcePage=ds;
	}
	
	this.transformCorpusXML = function(xmltxt)
	{	

			var iosvc = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
			var aboutBlankURI = iosvc.newURI("about:blank", null, null); 
			var parser = new DOMParser(null,aboutBlankURI,null);
            var xmlDocument = parser.parseFromString(xmltxt, "text/xml");
			var roottag = xmlDocument.documentElement;
		
			//Detection erreur du Parser
			if ((roottag.tagName == "parserError") || (roottag.namespaceURI == "http://www.mozilla.org/newlayout/xml/parsererror.xml"))
			{
				alert("Erreur XML");
			}
			
			//Get corpus number pages and word
			var corpusNode = xmlDocument.firstChild;
			tymining_DataController.nbPages = corpusNode.getAttribute("p");
			tymining_DataController.nbWords = corpusNode.getAttribute("w");
			tymining_DataController.nbWordsTotal = corpusNode.getAttribute("wt");
			tymining_UIController.setCorpusInfo();
			
			
            var xsltProcessor = new XSLTProcessor();
            xsltProcessor.importStylesheet(this.corpusXSL);
			var resultDocument = xsltProcessor.transformToDocument(xmlDocument);
			

			/* on transforme le doc XML en une string */
			var str = new XMLSerializer().serializeToString(resultDocument);
			var ds = tymining_DataSourceController.createDataSource(str);
			tymining_DataSourceController.assignDataSource("tymining_corpustree",ds,this.assignedDatasourceCorpus);
			this.assignedDatasourceCorpus=ds;
	}
}


//DataController
function tymining_DataController()
{
	this.idPage = 0;
	this.pageList = new Array();
	this.langList = new Array();
	
	//Corpus Info
	this.nbPages = "";
	this.nbWords = "";
	this.nbWordsTotal = "";
	
	this.addPage = function(pageUrl)
	{
		pageUrl = ""+pageUrl;
		pageUrl = pageUrl.replace(/&/g, "&amp;");
		var pageId = this.idPage;
		var page = new TyminingPage(this.idPage,pageUrl);
		
		this.pageList.push(page);
		this.idPage++;
		
		tymining_XPCOMController.addPage(page);
		
		return pageId;
	}
	
	this.addLang = function(lang)
	{
		this.langList.push(lang);
	}
	
	this.hasLang = function(lang)
	{
		for(l in this.langList)
		{
			if(this.langList[l]==lang)
				return true;
		}
		return false;
	}
	
	this.getPage = function(pageId)
	{
		return this.pageList[pageId];
	}
	
	this.resetCorpusInfo = function()
	{
		this.nbPages = "";
		this.nbWords = "";
		this.nbWordsTotal = "";
	}
}


//UI Controller
function tymining_UIController()
{
	this.currentPage=0;
	this.progress=false;
	this.progress2=false;
	this.langAuto = true;
	this.ui = false;
	
	this.getSidebar = function()
	{
		var sidebarWindow = document.getElementById("sidebar").contentWindow;
		if (sidebarWindow.location.href =="chrome://tymining/content/tymining.xul") {
			return sidebarWindow;
		}
	}
	
	this.startBtn = function()
	{
		if(tymining_RunController.IS_RUNNING())
		{
			tymining_RunController.STOP();
		}
		else if(tymining_RunController.IS_INITED())
		{
			tymining_RunController.START();
		}
		else
		{
			tymining_RunController.INIT();
		}
		
		tymining_UIController.initStartStatus();
	}
	
	this.initStartStatus = function()
	{
		if(tymining_RunController.IS_RUNNING())
		{
			this.getSidebar().document.getElementById("tymining_startBtn").setAttribute("label",tymining_Locale.GetStringFromName("tymining_ui_stop"));
			this.getSidebar().document.getElementById("tymining_resetBtn").setAttribute("disabled", true);
			this.getSidebar().document.getElementById("tymining_startBtn").setAttribute("disabled",false);
			
			//Export
			this.getSidebar().document.getElementById("tymining_exportBtn_exportPageCSV").setAttribute("disabled",true);
			this.getSidebar().document.getElementById("tymining_exportBtn_exportCorpusCSV").setAttribute("disabled",true);
			this.getSidebar().document.getElementById("tymining_exportBtn_exportGraphwordGDF").setAttribute("disabled",true);
			this.getSidebar().document.getElementById("tymining_exportBtn_exportGraphwordGEXF").setAttribute("disabled",true);
		}
		else if(tymining_RunController.IS_INITING())
		{
			this.getSidebar().document.getElementById("tymining_startBtn").setAttribute("label",tymining_Locale.GetStringFromName("tymining_ui_init"));
			this.getSidebar().document.getElementById("tymining_startBtn").setAttribute("disabled",true);
			this.getSidebar().document.getElementById("tymining_resetBtn").setAttribute("disabled", true);
			
			//Export
			this.getSidebar().document.getElementById("tymining_exportBtn_exportPageCSV").setAttribute("disabled",false);
			this.getSidebar().document.getElementById("tymining_exportBtn_exportCorpusCSV").setAttribute("disabled",false);
			this.getSidebar().document.getElementById("tymining_exportBtn_exportGraphwordGDF").setAttribute("disabled",false);
			this.getSidebar().document.getElementById("tymining_exportBtn_exportGraphwordGEXF").setAttribute("disabled",false);
		}
		else //Stopped
		{
			this.getSidebar().document.getElementById("tymining_startBtn").setAttribute("label",tymining_Locale.GetStringFromName("tymining_ui_start"));
			this.getSidebar().document.getElementById("tymining_startBtn").setAttribute("disabled",false);
			this.getSidebar().document.getElementById("tymining_resetBtn").setAttribute("disabled", false);
			
			//Export
			this.getSidebar().document.getElementById("tymining_exportBtn_exportPageCSV").setAttribute("disabled",false);
			this.getSidebar().document.getElementById("tymining_exportBtn_exportCorpusCSV").setAttribute("disabled",false);
			this.getSidebar().document.getElementById("tymining_exportBtn_exportGraphwordGDF").setAttribute("disabled",false);
			this.getSidebar().document.getElementById("tymining_exportBtn_exportGraphwordGEXF").setAttribute("disabled",false);
		}
	}
	
	this.resetBtn =function()
	{
		this.getSidebar().document.getElementById("tymining_resetBtn").setAttribute("disabled", true);
		tymining_DataSourceController.removeDataSource("tymining_pagetree", tymining_DataSourceController.assignedDatasourcePage);
		tymining_DataSourceController.removeDataSource("tymining_corpustree", tymining_DataSourceController.assignedDatasourceCorpus);
		this.currentPage=0;
		this.initUI();
		tymining_XPCOMController.reset();
	}
	
	this.initUI = function()
	{
		this.ui = true;
	
		//Log mode
		var logMode = tymining_PreferencesController.prefs.getBoolPref("extensions.tymining.log");
		this.getSidebar().document.getElementById("tymining_logTab").setAttribute("hidden", !logMode);
	
		//Control
		this.getSidebar().document.getElementById("tymining_startBtn").setAttribute("label",tymining_Locale.GetStringFromName("tymining_ui_start"));
		this.getSidebar().document.getElementById("tymining_resetBtn").setAttribute("disabled", true);
			
		//Export
		this.getSidebar().document.getElementById("tymining_exportBtn_exportPageCSV").setAttribute("disabled", true);
		this.getSidebar().document.getElementById("tymining_exportBtn_exportCorpusCSV").setAttribute("disabled", true);
		this.getSidebar().document.getElementById("tymining_exportBtn_exportGraphwordGDF").setAttribute("disabled", true);
		this.getSidebar().document.getElementById("tymining_exportBtn_exportGraphwordGEXF").setAttribute("disabled", true);
		
		//Progress
		this.getSidebar().document.getElementById("tymining_pageprogress").setAttribute("hidden", true);
		
		//Page
		this.getSidebar().document.getElementById("tymining_page_valid").setAttribute("checked",false);
		this.getSidebar().document.getElementById("tymining_page_lang").setAttribute("value","");
		this.getSidebar().document.getElementById("tymining_page_url").setAttribute("value","");
		
		//Word graph
		if(tymining_PreferencesController.prefs.getBoolPref("extensions.tymining.EnableWordGraph"))
		{
			this.getSidebar().document.getElementById("tymining_exportBox_wordgraph").setAttribute("hidden",false);
			this.getSidebar().document.getElementById("tymining_options_wordgraph").setAttribute("hidden",false);
		}
		else
		{
			this.getSidebar().document.getElementById("tymining_exportBox_wordgraph").setAttribute("hidden",true);
			this.getSidebar().document.getElementById("tymining_options_wordgraph").setAttribute("hidden",true);
		}
		
		//Options
		var minOccTextbox = this.getSidebar().document.getElementById("tymining_option_minOcc");
		minOccTextbox.value = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.CorpusMinWords");
		var nminTextBox = this.getSidebar().document.getElementById("tymining_option_wordgraph_nmin");
		nminTextBox.value = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.wordgraph.wordRepeatMinimum");
		var pminTextBox = this.getSidebar().document.getElementById("tymining_option_wordgraph_pmin");
		pminTextBox.value = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.wordgraph.pageRepeatMinimum");
		var nbwordmaxTextBox = this.getSidebar().document.getElementById("tymining_option_wordgraph_nbwordmax");
		nbwordmaxTextBox.value = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.wordgraph.nbWordPerPageMaximum");
		
		//Corpus Info
		tymining_UIController.setCorpusInfo();
		
		//Page info
		if(!this.currentPage==0)
			tymining_UIController.refreshPage(this.currentPage);
		
		//Additional init
		tymining_UIController.initStartStatus();
		tymining_UIController.initLangOptions();
		
		//Progress
		if(tymining_XPCOMController.isAnalysing())
			tymining_UIController.enableAnalysis();
		else
			tymining_UIController.disableAnalysis();
			
		if(this.progress2==true)
			tymining_UIController.startProgress();
	}
	
	this.closeUI = function()
	{
		this.ui = false;
	}
	
	this.initLangOptions = function()
	{
		//Options
		var langMenuPopup = this.getSidebar().document.getElementById("tymining_option_defaultLang");
		var langMenuList = this.getSidebar().document.getElementById("tymining_option_defaultLangList");
		var defaultStr = tymining_Locale.GetStringFromName("tymining_lang_default");
		
		//Reset
		while(langMenuPopup.hasChildNodes())
		{
			langMenuPopup.removeChild(langMenuPopup.firstChild);
		}
		
		//Auto
		var menuitemAuto = document.createElement("menuitem");
		var localeLangAuto = tymining_Locale.GetStringFromName("tymining_lang_auto");
		menuitemAuto.setAttribute("label", localeLangAuto);
		langMenuPopup.appendChild(menuitemAuto);
		
		var c=0;
		for(l in tymining_DataController.langList)
		{
			var lang = tymining_DataController.langList[l];
			LOG(""+lang);
			var localeLang = tymining_Locale.GetStringFromName("tymining_lang_"+lang);
			var menuitem = document.createElement("menuitem");
			menuitem.setAttribute("label", localeLang+" ("+defaultStr+")");
			langMenuPopup.appendChild(menuitem);
			
			if(!this.langAuto && this.defaultLang==lang)
				langMenuList.selectedIndex=c+1;
				
			c++
		}
		
		if(this.langAuto)
			langMenuList.selectedIndex=0;		
	}
	
	this.refreshPage = function(pageObject)
	{
		this.getSidebar().document.getElementById("tymining_page_url").setAttribute("value",pageObject.url);
		
		//Lang locale
		var lang = this.getLocaleLang(pageObject.lang);
		this.getSidebar().document.getElementById("tymining_page_lang").setAttribute("value",lang);
		
		//State
		if(pageObject.state==STATE_GREEN)
			this.getSidebar().document.getElementById("tymining_page_valid").setAttribute("checked",true);
		else
			this.getSidebar().document.getElementById("tymining_page_valid").setAttribute("checked",false);
		
		//Current page
		this.currentPage=pageObject;
		
	}
	
	this.refreshCorpus = function()
	{
	
	}
	
	this.getLocaleLang = function(lang)
	{
		if(tymining_DataController.hasLang(lang))
		{
			var str = tymining_Locale.GetStringFromName("tymining_lang_"+lang);
			if(!this.langAuto)
				str += " (" + tymining_Locale.GetStringFromName("tymining_lang_default") + ")";
			return str;
		}
		else
			return "?";
	}
	
	this.enableAnalysis = function()
	{
		if(tymining_UIController.ui)
			this.getSidebar().document.getElementById("tymining_pageprogress").setAttribute("hidden", "false");
		this.progress=true;
		tymining_RunController.COMPUTING(true);
	}
	
	this.disableAnalysis = function()
	{
		if(tymining_UIController.ui)
			this.getSidebar().document.getElementById("tymining_pageprogress").setAttribute("hidden", "true");
		this.progress=false;
		tymining_RunController.COMPUTING(false);
	}
	
	this.startProgress = function()
	{
		if(tymining_UIController.ui)
		{
			this.getSidebar().document.getElementById("tymining_progress").value = 0;
			this.getSidebar().document.getElementById("tymining_progress").setAttribute("hidden", "false");
		}
		
		this.progress2=true;
		tymining_RunController.COMPUTING(true);
	}
	
	this.setProgress = function(progress)
	{
		if(this.progress2==false)
			tymining_UIController.startProgress();
			
		if(tymining_UIController.ui)
			this.getSidebar().document.getElementById("tymining_progress").value = progress;
		
		//if(progress==100)
			//tymining_UIController.stopProgress();
	}
	
	this.stopProgress = function()
	{
		if(tymining_UIController.ui)
			this.getSidebar().document.getElementById("tymining_progress").setAttribute("hidden", "true");
			
		this.progress2=false;
		tymining_RunController.COMPUTING(false);
	}
	
	this.validPageCheckbox = function()
	{
		if(this.currentPage!=0)
		{
			var checked = this.getSidebar().document.getElementById("tymining_page_valid").getAttribute("checked");
		if(checked)
			this.currentPage.state = STATE_GREEN;
		else
			this.currentPage.state = STATE_RED;
		tymining_XPCOMController.setPageState(this.currentPage);
		}
	}
	
	this.optionDefaultLang = function()
	{
		var langMenuList = this.getSidebar().document.getElementById("tymining_option_defaultLangList");
		var index = langMenuList.selectedIndex;
		
		if(index==0)
		{
			this.langAuto = true;
		}
		else
		{
			this.langAuto = false;
			this.defaultLang = tymining_DataController.langList[index-1];
		}
	}
	
	this.setMinOcc = function()
	{
		var minOccTextbox = this.getSidebar().document.getElementById("tymining_option_minOcc");
		var minOccValue = minOccTextbox.value;
		tymining_PreferencesController.prefs.setIntPref("extensions.tymining.CorpusMinWords", minOccValue);
	}
	
	this.setCorpusInfo = function()
	{
		var nbPageDesc = this.getSidebar().document.getElementById("tymining_corpus_nbpages");
		nbPageDesc.setAttribute("value", tymining_DataController.nbPages);
		
		var nbWordDesc = this.getSidebar().document.getElementById("tymining_corpus_nbexp");
		nbWordDesc.setAttribute("value", tymining_DataController.nbWords);
		
		var nbWordTotalDesc = this.getSidebar().document.getElementById("tymining_corpus_nbexp_total");
		nbWordTotalDesc.setAttribute("value", tymining_DataController.nbWordsTotal);
		
	}
	
	this.setWGNbWordMax = function()
	{
		var nbwordmaxTextBox = this.getSidebar().document.getElementById("tymining_option_wordgraph_nbwordmax");
		var nbwordmaxValue = nbwordmaxTextBox.value;
		tymining_PreferencesController.prefs.setIntPref("extensions.tymining.wordgraph.nbWordPerPageMaximum", nbwordmaxValue);
	}
	
	this.setWGNmin = function()
	{
		var nminTextBox = this.getSidebar().document.getElementById("tymining_option_wordgraph_nmin");
		var nminValue = nminTextBox.value;
		tymining_PreferencesController.prefs.setIntPref("extensions.tymining.wordgraph.wordRepeatMinimum", nminValue);
	}
	
	this.setWGPmin = function()
	{
		var pminTextBox = this.getSidebar().document.getElementById("tymining_option_wordgraph_pmin");
		var pminValue = pminTextBox.value;
		tymining_PreferencesController.prefs.setIntPref("extensions.tymining.wordgraph.pageRepeatMinimum", pminValue);
	}
	
	this.appendLog = function(text)
	{
		var logMode = tymining_PreferencesController.prefs.getBoolPref("extensions.tymining.log");
		if(logMode)
		{
			var logTextBox = this.getSidebar().document.getElementById("tymining_logbox");
			logTextBox.value=logTextBox.value+text+"\n";
		}
	}
	
	this.resetLog = function()
	{
		var logTextBox = this.getSidebar().document.getElementById("tymining_logbox");
		logTextBox.value="";
	}
	
	this.openStopListDirectory = function()
	{
		var urlStr = tymining_getExtensionProfileLocation("components\\stopwords");
		gBrowser.addTab(urlStr);
	}
	
	this.openWordGraphHelpProcess = function()
	{
		gBrowser.addTab("http://timmyminer.mozdev.org/#expnetwork");
	}
	
	this.openWordGraphHelpVisualize = function()
	{
		gBrowser.addTab("http://gephi.org");
	}
	
} 

//Export Controller
function tymining_ExportController()
{
	this.pageXSL = 0;
	this.corpusXSL = 0;
	this.currentExport = 0;
	this.initXSLT = function()
	{
		//XSLT doc page
		var xsltDoc = document.implementation.createDocument("", "test", null);
		this.pageXSL = xsltDoc;
		xsltDoc.addEventListener("load", tymining_DataSourceController.initXSLTFinished, false);
		xsltDoc.load("chrome://tymining/content/pageToCSV.xsl");
		
		//XSLT doc corpus
		var xsltDoc2 = document.implementation.createDocument("", "test", null);
		this.corpusXSL = xsltDoc2;
		xsltDoc2.addEventListener("load", tymining_DataSourceController.initXSLTFinished, false);
		xsltDoc2.load("chrome://tymining/content/corpusToCSV.xsl");
	}
	
	this.initXSLTFinished = function()
	{
		
	}
	
	this.transformInCSV = function(xmltxt, slylesheet)
	{	

		var iosvc = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var aboutBlankURI = iosvc.newURI("about:blank", null, null); 
		var parser = new DOMParser(null,aboutBlankURI,null);
		var xmlDocument = parser.parseFromString(xmltxt, "text/xml");
		var roottag = xmlDocument.documentElement;
	
		//Detection erreur du Parser
		if ((roottag.tagName == "parserError") || (roottag.namespaceURI == "http://www.mozilla.org/newlayout/xml/parsererror.xml"))
		{
			tymining_ErrorController.Severe("Parse error on the XMl output");
		}
		
		var xsltProcessor = new XSLTProcessor();
		xsltProcessor.importStylesheet(slylesheet);
		var resultDocument = xsltProcessor.transformToDocument(xmlDocument);
		
		/* on transforme le doc XML en une string */
		var str = new XMLSerializer().serializeToString(resultDocument);
		return str;
	}
	
	this.exportPageCSV = function()
	{
		//Get page XML
		var result = {value: null};
		tymining_XPCOMController.compo.getPageCorpus(tymining_UIController.currentPage.id, -1, result);
		var xmlResult = result.value;
	
		var exportTxt= this.transformInCSV(xmlResult, this.pageXSL);
		exportTxt= exportTxt.replace("<transformiix:result xmlns:transformiix=\"http://www.mozilla.org/TransforMiix\">","");
		exportTxt= exportTxt.replace("</transformiix:result>","");
		
		tymining_saveDialog(exportTxt, "list");
	}
	
	this.exportCorpusCSV = function()
	{
		//Get corpus XML
		var result = {value: null};
		tymining_XPCOMController.compo.getAllCorpus(-1, result);
		var xmlResult = result.value;
	
		var exportTxt= this.transformInCSV(xmlResult, this.corpusXSL);
		exportTxt= exportTxt.replace("<transformiix:result xmlns:transformiix=\"http://www.mozilla.org/TransforMiix\">","");
		exportTxt= exportTxt.replace("</transformiix:result>","");
		
		tymining_saveDialog(exportTxt, "list");
	}
	
	this.exportWordGraphGDF = function()
	{
		//Set wordgraph options
		var pmin = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.wordgraph.pageRepeatMinimum");
		var nmin = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.wordgraph.wordRepeatMinimum");
		var nbwordmax = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.wordgraph.nbWordPerPageMaximum");
		tymining_XPCOMController.compo.setWordGraphNMin(nmin);
		tymining_XPCOMController.compo.setWordGraphPMin(pmin);
		tymining_XPCOMController.compo.setNbWordPerPage(nbwordmax);
		
		this.currentExport="GDF";
		tymining_UIController.startProgress();
		tymining_XPCOMController.compo.refreshWordGraph();
	}
	
	this.exportWordGraphGDFStep2 = function()
	{
		//Get corpus graph XML
		var result = {value: null};
		tymining_XPCOMController.compo.getWordGraph(result);
		var xmlResult = result.value;
		//LOG(xmlResult);
		
		var exportTxt= this.buildGDF(xmlResult);
		tymining_saveDialog(exportTxt, "graph");
	}
	
	this.exportWordGraphGEXF = function()
	{
		//Set wordgraph options
		var pmin = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.wordgraph.pageRepeatMinimum");
		var nmin = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.wordgraph.wordRepeatMinimum");
		var nbwordmax = tymining_PreferencesController.prefs.getIntPref("extensions.tymining.wordgraph.nbWordPerPageMaximum");
		tymining_XPCOMController.compo.setWordGraphNMin(nmin);
		tymining_XPCOMController.compo.setWordGraphPMin(pmin);
		tymining_XPCOMController.compo.setNbWordPerPage(nbwordmax);
		
		this.currentExport="GEXF";
		tymining_UIController.startProgress();
		tymining_XPCOMController.compo.refreshWordGraph();
	}
	
	this.exportWordGraphGEXFStep2 = function()
	{
		//Get corpus graph XML
		var result = {value: null};
		tymining_XPCOMController.compo.getWordGraph(result);
		var xmlResult = result.value;
		//LOG(xmlResult);
		
		var exportTxt= this.buildGEXF(xmlResult);
		tymining_saveDialog(exportTxt, "graph");
	}
	
	this.exportWordGraphStep2 = function()
	{
		if(this.currentExport=="GDF")
		{
			tymining_UIController.stopProgress();
			this.exportWordGraphGDFStep2();
		}
		else if(this.currentExport=="GEXF")
		{
			tymining_UIController.stopProgress();
			this.exportWordGraphGEXFStep2();
		}
	}
	
	this.buildGEXF = function(xmltxt)
	{
		var nodesText = "";
		var edgesText = "";
		
		var nodeArray = new Array();
		
		var iosvc = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var aboutBlankURI = iosvc.newURI("about:blank", null, null); 
		var parser = new DOMParser(null,aboutBlankURI,null);
		var xmlDocument = parser.parseFromString(xmltxt, "text/xml");
		var roottag = xmlDocument.documentElement;
	
		//Detection erreur du Parser
		if ((roottag.tagName == "parserError") || (roottag.namespaceURI == "http://www.mozilla.org/newlayout/xml/parsererror.xml"))
		{
			tymining_ErrorController.Severe("Parse error on the XML output");
		}
		
		var wordGraphNode = xmlDocument.firstChild;
		var nodeId=0;
		
		var tab = wordGraphNode.getElementsByTagName("c");
		for(var i=0;i < tab.length; i++)
		{
			var pairNode = tab[i];
			var fVal = ""+pairNode.getAttribute("f");
			var sVal = ""+pairNode.getAttribute("s");
			var fValId = -1;
			var sValId = -1;
			
			//F
			if(nodeArray[fVal] != null)
			{
				//The word already exist
				var oldExp = nodeArray[fVal];
				fValId = oldExp.id;
			}
			else
			{
				var newExp = new WGExpression(nodeId,fVal);
				nodeId++;
				nodeArray[fVal] = newExp;
				fValId = newExp.id;
				
				nodesText += "\t\t\t<node id=\""+fValId+"\" label=\""+fVal+"\" />\n";
			}
			
			
			//S
			if(nodeArray[sVal] != null)
			{
				//The word already exist
				var oldExp = nodeArray[sVal];
				sValId = oldExp.id;
			}
			else
			{
				var newExp = new WGExpression(nodeId,sVal);
				nodeId++;
				nodeArray[sVal] = newExp;
				sValId = newExp.id;
				
				nodesText += "\t\t\t<node id=\""+sValId+"\" label=\""+sVal+"\" />\n";
			}
			
			//New edge
			if(fValId!=-1 && sValId!=-1)
				edgesText += "\t\t\t<edge source=\""+fValId+"\" target=\""+sValId+"\" />\n";
			
		}
		
		
		//Build GEXF
		var gexfText = "<xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
		gexfText += "\t<gexf xmlns=\"http://www.gephi.org/gexf\">\n";
		gexfText += "\t<graph type=\"static\" edges=\"simple\">\n";
		gexfText += "\t\t<attributes type=\"static\" />\n";
		gexfText += "\t\t<nodes>\n";
		gexfText += nodesText;
		gexfText += "\t\t</nodes>\n\t\t<edges>\n";
		gexfText += edgesText;
		gexfText += "\t\t</edges>\n\t</graph>\n</gexf>";
		
		return gexfText;
	
	}
	
	this.buildGDF = function(xmltxt)
	{	
		var nodesText = "";
		var edgesText = "";
		
		var nodeArray = new Array();
		
		var iosvc = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
		var aboutBlankURI = iosvc.newURI("about:blank", null, null); 
		var parser = new DOMParser(null,aboutBlankURI,null);
		var xmlDocument = parser.parseFromString(xmltxt, "text/xml");
		var roottag = xmlDocument.documentElement;
	
		//Detection erreur du Parser
		if ((roottag.tagName == "parserError") || (roottag.namespaceURI == "http://www.mozilla.org/newlayout/xml/parsererror.xml"))
		{
			tymining_ErrorController.Severe("Parse error on the XML output");
		}
		
		var wordGraphNode = xmlDocument.firstChild;
		var nodeId=0;
		
		var tab = wordGraphNode.getElementsByTagName("c");
		for(var i=0;i < tab.length; i++)
		{
			var pairNode = tab[i];
			var fVal = ""+pairNode.getAttribute("f");
			var sVal = ""+pairNode.getAttribute("s");
			var fValId = -1;
			var sValId = -1;
			
			//F
			if(nodeArray[fVal] != null)
			{
				//The word already exist
				var oldExp = nodeArray[fVal];
				fValId = oldExp.id;
			}
			else
			{
				var newExp = new WGExpression("n"+nodeId,fVal);
				nodeId++;
				nodeArray[fVal] = newExp;
				fValId = newExp.id;
				
				nodesText += fValId+","+"\""+fVal+"\""+"\n";
			}
			
			
			//S
			if(nodeArray[sVal] != null)
			{
				//The word already exist
				var oldExp = nodeArray[sVal];
				sValId = oldExp.id;
			}
			else
			{
				var newExp = new WGExpression("n"+nodeId,sVal);
				nodeId++;
				nodeArray[sVal] = newExp;
				sValId = newExp.id;
				
				nodesText += sValId+","+"\""+sVal+"\""+"\n";
			}
			
			//New edge
			if(fValId!=-1 && sValId!=-1)
				edgesText += fValId+","+sValId+"\n";
			
		}
		
		
		//Build GDF
		var gdfText = "";
		gdfText += "nodedef>name VARCHAR,label VARCHAR"+"\n";
		gdfText += nodesText;
		gdfText += "edgedef>node1 VARCHAR,node2 VARCHAR"+"\n";
		gdfText += edgesText;
		
		return gdfText;
	
	}
}

function WGExpression(_id,_exp)
{
	this.id = _id;
	this.exp = _exp;
}

//Error Controller
function tymining_ErrorController()
{
	this.Warning = function(message)
	{
		alert("Warning : "+message);
	}
	this.Severe = function(message)
	{
		alert("Severe : "+message);
	}
}
	
//Run Controller
function tymining_RunController()
{
	this.running = false;
	this.inited = false;
	this.initing = false;
	this.computing = false;
	
	this.INIT = function(){
		this.running = true;
		tymining_XPCOMController.initComponent();
		tymining_DataSourceController.initXSLT();
		tymining_ExportController.initXSLT();
	}
	this.START = function(){
		this.running = true;
		gBrowser.addEventListener("load", tymining_onContentLoaded, true);
		tymining_RunController.updateButtonStyle();
	}
	this.STOP = function(){
		this.running = false;
		gBrowser.removeEventListener("load", tymining_onContentLoaded, true);
		tymining_RunController.updateButtonStyle();
	}
	this.IS_RUNNING = function(){
		return this.running && this.inited;
	}
	this.IS_INITED = function(){
		return this.inited;
	}
	
	this.IS_COMPUTING = function()
	{
		return this.running && this.inited && this.computing;
	}
	
	this.INITED = function(){
		this.running = false;
		this.inited=true;
	}
	
	this.COMPUTING = function(compute)
	{
		this.computing=compute;
		tymining_RunController.updateButtonStyle();
	}
	
	this.IS_INITING = function(){
		return this.running && !this.inited;
	}
	
	this.updateButtonStyle = function()
	{
		var button = document.getElementById("tymining-startButton");
		var color = "white";
		
		if(tymining_RunController.IS_COMPUTING())
			color="blue";
		else if(tymining_RunController.IS_RUNNING())
			color="green";
		else if(tymining_RunController.IS_INITED())
			color="red";
		
			
		button.setAttribute("style", "height:20px;list-style-image: url('chrome://tymining/skin/bullet_"+color+".png');");
	}
}

//Preferences Controller
function tymining_PreferencesController()
{
	this.prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
	this.getString = function(pname){
		var prefs = this.prefs.getBranch("extensions.tymining.");
		return prefs.getComplexValue(pname, Components.interfaces.nsISupportsString).data;
	}
	this.setString = function(pname, value){
		var prefs = this.prefs.getBranch("extensions.tymining.");
		prefs.setCharPref(pname, value);
	}
}

/*+--------------------------------------------------------------------+
    |			      	IO				|
    +--------------------------------------------------------------------+*/

function tymining_saveDialog(textContent, type)
{
	var nsIFilePicker = Components.interfaces.nsIFilePicker;
	var filePickerSave = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
	filePickerSave.init(window, tymining_Locale.GetStringFromName("tymining_saveFile"),nsIFilePicker.modeSave);
	
	if(type == "list")
		filePickerSave.appendFilter(tymining_Locale.GetStringFromName("tymining_dataFile_list")+" (*.csv;*.txt)","*.csv; *.txt");
	else if(type == "graph")
		filePickerSave.appendFilter(tymining_Locale.GetStringFromName("tymining_dataFile_graph")+" (*.gdf;*.gexf)","*.gdf; *.gexf");
	
	var res = filePickerSave.show();
	if (res == nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace )
	{
		var thefile = filePickerSave.file;
		tymining_saveFile(thefile,textContent);
	}
}
	
function tymining_saveFile(file, content)
{
	try {
		netscape.security.PrivilegeManager.enablePrivilege("UniversalXPConnect");
	} catch (e) {
		alert("Permission to save file was denied.");
	}

	if ( file.exists() == false ) {
		file.create( Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 420 );
	}

	var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance( Components.interfaces.nsIFileOutputStream );
	var cos= Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);

	
	/* Open flags 
	#define PR_RDONLY       0x01
	#define PR_WRONLY       0x02
	#define PR_RDWR         0x04
	#define PR_CREATE_FILE  0x08
	#define PR_APPEND      0x10
	#define PR_TRUNCATE     0x20
	#define PR_SYNC         0x40
	#define PR_EXCL         0x80
	*/
	/*
	** File modes ....
	**
	** CAVEAT: 'mode' is currently only applicable on UNIX platforms.
	** The 'mode' argument may be ignored by PR_Open on other platforms.
	**
	**   00400   Read by owner.
	**   00200   Write by owner.
	**   00100   Execute (search if a directory) by owner.
	**   00040   Read by group.
	**   00020   Write by group.
	**   00010   Execute by group.
	**   00004   Read by others.
	**   00002   Write by others
	**   00001   Execute by others.
	**
	*/
	
	var charset = "UTF-8";
	
	outputStream.init( file, 0x02 | 0x08 | 0x20, 420, 0 );
	cos.init(outputStream, charset, 0, " ".charCodeAt(0));
	cos.writeString(content);

	/*var written = outputStream.write( content, content.length );
	if(written!=content.length)
		alert("Erreur d'criture");*/
		
	cos.close();
	outputStream.close();
}

/*+--------------------------------------------------------------------+
    |			      	UTILS				|
    +--------------------------------------------------------------------+*/

//Get the path of the extension on the filesystem, plus the given subpath. Manager GNU and Windows paths
function tymining_getExtensionProfileLocation(subpath)
{
	var pathservice = Components.classes["@mozilla.org/file/directory_service;1"].getService( Components.interfaces.nsIProperties);
	var path = pathservice.get("ProfD", Components.interfaces.nsIFile).path;
	if(path.indexOf("\\")==2)
	{
		//Windows path
		path+="\\extensions\\tymining@web-mining.fr\\";
		path+=subpath;
	}
	else
	{
		//Gnu path
		path+="/extensions/tymining@web-mining.fr/";
		subpath=subpath.replace(/\\/g,"\/" ); 
		path+=subpath;
	}
	return path;
}

//Log in Javascript console
function LOG(msg) {
  var consoleService = Components.classes["@mozilla.org/consoleservice;1"]
                                 .getService(Components.interfaces.nsIConsoleService);
  consoleService.logStringMessage(msg);
}
